#
# Copyright (c) 2008-2016 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from datetime import datetime

# import interfaces
from pyams_content.interfaces import CREATE_CONTENT_PERMISSION
from pyams_content.shared.blog.interfaces import IWfBlogPost, IBlogManager
from pyams_i18n.interfaces import II18n, II18nManager
from pyams_skin.interfaces.viewlet import IMenuHeader, IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowVersions, IWorkflowInfo
from pyams_zmi.interfaces.menu import IContentManagementMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_content.shared.blog.manager import BlogFolder
from pyams_content.shared.common.zmi import SharedContentAddForm, SharedContentAJAXAddForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces import IContentTitle
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter, ContextRequestAdapter
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyramid.view import view_config
from zope.interface import Interface
from zope.lifecycleevent import ObjectCreatedEvent

from pyams_content import _


@adapter_config(context=(IWfBlogPost, IContentManagementMenu), provides=IMenuHeader)
class BlogPostContentMenuHeader(ContextRequestAdapter):
    """Blog post content menu header adapter"""

    header = _("This blog post")


@adapter_config(context=(IWfBlogPost, IPyAMSLayer, Interface), provides=IContentTitle)
class BlogPostTitleAdapter(ContextRequestViewAdapter):
    """Blog post title adapter"""

    @property
    def title(self):
        translate = self.request.localizer.translate
        return translate(_("Blog post « {title} »")).format(
            title=II18n(self.context).query_attribute('title', request=self.request))


@viewlet_config(name='add-shared-content.action', context=IBlogManager, layer=IAdminLayer, view=Interface,
                manager=IWidgetTitleViewletManager, permission=CREATE_CONTENT_PERMISSION, weight=1)
class BlogPostAddAction(ToolbarAction):
    """Blog post adding action"""

    label = _("Add blog post")
    url = 'add-shared-content.html'
    modal_target = True


@pagelet_config(name='add-shared-content.html', context=IBlogManager, layer=IPyAMSLayer,
                permission=CREATE_CONTENT_PERMISSION)
class BlogPostAddForm(SharedContentAddForm):
    """Blog post add form"""

    legend = _("Add blog post")
    content_url = None

    def add(self, wf_content):
        # create shared content
        content = self.context.shared_content_factory()
        self.request.registry.notify(ObjectCreatedEvent(content))
        # check blog folders
        now = datetime.utcnow()
        year, month = now.strftime('%Y:%m').split(':')
        year_folder = self.context.get(year)
        if year_folder is None:
            year_folder = self.context[year] = BlogFolder()
        month_folder = year_folder.get(month)
        if month_folder is None:
            month_folder = year_folder[month] = BlogFolder()
        # check title language
        added = False
        i18n_manager = II18nManager(self.context)
        for lang in i18n_manager.get_languages():
            title = translate_string(wf_content.title.get(lang))
            if title:
                content_name = translate_string(title, force_lower=True, spaces='-')
                month_folder[content_name] = content
                added = True
                break
        # handle workflow
        if added:
            IWorkflowVersions(content).add_version(wf_content, None)
            IWorkflowInfo(wf_content).fire_transition('init')
            self.content_url = absolute_url(wf_content, self.request, 'admin')

    def nextURL(self):
        return self.content_url


@view_config(name='add-shared-content.json', context=IBlogManager, request_type=IPyAMSLayer,
             permission=CREATE_CONTENT_PERMISSION, renderer='json', xhr=True)
class BlogPostAJAXAddForm(SharedContentAJAXAddForm, BlogPostAddForm):
    """Blog post add form, JSON renderer"""
